import 'reflect-metadata';
import { ObjectDefinitionOptions } from '..';
export declare type decoratorKey = string | symbol;
export declare const PRELOAD_MODULE_KEY = "INJECTION_PRELOAD_MODULE_KEY";
export declare class DecoratorManager extends Map {
    /**
     * the key for meta data store in class
     */
    injectClassKeyPrefix: string;
    /**
     * the key for method meta data store in class
     */
    injectClassMethodKeyPrefix: string;
    /**
     * the key for method meta data store in method
     */
    injectMethodKeyPrefix: string;
    saveModule(key: any, module: any): void;
    static getDecoratorClassKey(decoratorNameKey: decoratorKey): string;
    static getDecoratorMethodKey(decoratorNameKey: decoratorKey): string;
    static getDecoratorClsMethodPrefix(decoratorNameKey: decoratorKey): string;
    static getDecoratorClsMethodKey(decoratorNameKey: decoratorKey, methodKey: decoratorKey): string;
    listModule(key: any): unknown[];
    static getOriginMetadata(metaKey: any, target: any, method?: any): any;
    /**
     * save meta data to class or property
     * @param decoratorNameKey the alias name for decorator
     * @param data the data you want to store
     * @param target target class
     * @param propertyName
     */
    saveMetadata(decoratorNameKey: decoratorKey, data: any, target: any, propertyName?: any): void;
    /**
     * attach data to class or property
     * @param decoratorNameKey
     * @param data
     * @param target
     * @param propertyName
     */
    attachMetadata(decoratorNameKey: decoratorKey, data: any, target: any, propertyName?: any): void;
    /**
     * get single data from class or property
     * @param decoratorNameKey
     * @param target
     * @param propertyName
     */
    getMetadata(decoratorNameKey: decoratorKey, target: any, propertyName?: any): any;
    /**
     * save property data to class
     * @param decoratorNameKey
     * @param data
     * @param target
     * @param propertyName
     */
    savePropertyDataToClass(decoratorNameKey: decoratorKey, data: any, target: any, propertyName: any): void;
    /**
     * attach property data to class
     * @param decoratorNameKey
     * @param data
     * @param target
     * @param propertyName
     */
    attachPropertyDataToClass(decoratorNameKey: decoratorKey, data: any, target: any, propertyName: any): void;
    /**
     * get property data from class
     * @param decoratorNameKey
     * @param target
     * @param propertyName
     */
    getPropertyDataFromClass(decoratorNameKey: decoratorKey, target: any, propertyName: any): any;
    /**
     * list property data from class
     * @param decoratorNameKey
     * @param target
     */
    listPropertyDataFromClass(decoratorNameKey: decoratorKey, target: any): any[];
}
/**
 * save data to class
 * @param decoratorNameKey
 * @param data
 * @param target
 */
export declare function saveClassMetadata(decoratorNameKey: decoratorKey, data: any, target: any): void;
/**
 * attach data to class
 * @param decoratorNameKey
 * @param data
 * @param target
 */
export declare function attachClassMetadata(decoratorNameKey: decoratorKey, data: any, target: any): void;
/**
 * get data from class
 * @param decoratorNameKey
 * @param target
 */
export declare function getClassMetadata(decoratorNameKey: decoratorKey, target: any): any;
/**
 * save method data to class
 * @deprecated
 * @param decoratorNameKey
 * @param data
 * @param target
 * @param method
 */
export declare function saveMethodDataToClass(decoratorNameKey: decoratorKey, data: any, target: any, method: any): void;
/**
 * attach method data to class
 * @deprecated
 * @param decoratorNameKey
 * @param data
 * @param target
 * @param method
 */
export declare function attachMethodDataToClass(decoratorNameKey: decoratorKey, data: any, target: any, method: any): void;
/**
 * get method data from class
 * @deprecated
 * @param decoratorNameKey
 * @param target
 * @param method
 */
export declare function getMethodDataFromClass(decoratorNameKey: decoratorKey, target: any, method: any): any;
/**
 * list method data from class
 * @deprecated
 * @param decoratorNameKey
 * @param target
 */
export declare function listMethodDataFromClass(decoratorNameKey: decoratorKey, target: any): any[];
/**
 * save method data
 * @deprecated
 * @param decoratorNameKey
 * @param data
 * @param target
 * @param method
 */
export declare function saveMethodMetadata(decoratorNameKey: decoratorKey, data: any, target: any, method: any): void;
/**
 * attach method data
 * @deprecated
 * @param decoratorNameKey
 * @param data
 * @param target
 * @param method
 */
export declare function attachMethodMetadata(decoratorNameKey: decoratorKey, data: any, target: any, method: any): void;
/**
 * get method data
 * @deprecated
 * @param decoratorNameKey
 * @param target
 * @param method
 */
export declare function getMethodMetadata(decoratorNameKey: decoratorKey, target: any, method: any): any;
/**
 * save property data to class
 * @param decoratorNameKey
 * @param data
 * @param target
 * @param propertyName
 */
export declare function savePropertyDataToClass(decoratorNameKey: decoratorKey, data: any, target: any, propertyName: any): void;
/**
 * attach property data to class
 * @param decoratorNameKey
 * @param data
 * @param target
 * @param propertyName
 */
export declare function attachPropertyDataToClass(decoratorNameKey: decoratorKey, data: any, target: any, propertyName: any): void;
/**
 * get property data from class
 * @param decoratorNameKey
 * @param target
 * @param propertyName
 */
export declare function getPropertyDataFromClass(decoratorNameKey: decoratorKey, target: any, propertyName: any): any;
/**
 * list property data from class
 * @param decoratorNameKey
 * @param target
 */
export declare function listPropertyDataFromClass(decoratorNameKey: decoratorKey, target: any): any[];
/**
 * save property data
 * @param decoratorNameKey
 * @param data
 * @param target
 * @param propertyName
 */
export declare function savePropertyMetadata(decoratorNameKey: decoratorKey, data: any, target: any, propertyName: any): void;
/**
 * attach property data
 * @param decoratorNameKey
 * @param data
 * @param target
 * @param propertyName
 */
export declare function attachPropertyMetadata(decoratorNameKey: decoratorKey, data: any, target: any, propertyName: any): void;
/**
 * get property data
 * @param decoratorNameKey
 * @param target
 * @param propertyName
 */
export declare function getPropertyMetadata(decoratorNameKey: decoratorKey, target: any, propertyName: any): any;
/**
 * save preload module by target
 * @param target
 */
export declare function savePreloadModule(target: any): void;
/**
 * list preload module
 */
export declare function listPreloadModule(): unknown[];
/**
 * save module to inner map
 * @param decoratorNameKey
 * @param target
 */
export declare function saveModule(decoratorNameKey: decoratorKey, target: any): void;
/**
 * list module from decorator key
 * @param decoratorNameKey
 */
export declare function listModule(decoratorNameKey: decoratorKey): unknown[];
/**
 * clear all module
 */
export declare function clearAllModule(): void;
/**
 * get parameter name from function
 * @param func
 */
export declare function getParamNames(func: any): any;
/**
 * get provider id from module
 * @param module
 */
export declare function getProviderId(module: any): string;
/**
 * get object definition metadata
 * @param module
 */
export declare function getObjectDefinition(module: any): ObjectDefinitionOptions;
